import logging
import sys
import json

import requests

from stevesie.config import CONFIG, API_TOKEN, BASE_HOST

BASE_URL = BASE_HOST + '/api/v1/'

logging.basicConfig(stream=sys.stdout, level=logging.INFO)

def get(url, params=None):
    params_log = params or ''
    logging.info('Fetching %s %s', url, params_log)
    response = requests.get(
        url,
        headers={'Token': API_TOKEN},
        params=params)

    if response.status_code == 401:
        raise Exception('Access Denied - Check your token or the resource URL')
    return response.json()

def post(url, params=None, query=None):
    query_log = query or ''
    logging.info('Posting %s %s', url, query_log)
    response = requests.post(
        url,
        headers={'Token': API_TOKEN},
        json=params,
        params=query)

    if response.status_code == 401:
        raise Exception('Access Denied - Check your token or the resource URL')

    try:
        return response.json(), response.status_code
    except json.decoder.JSONDecodeError:
        return response.text, response.status_code

def delete(url):
    logging.info('Deleting %s', url)
    requests.delete(url, headers={'Token': API_TOKEN})



name = 'stevesie'

from stevesie.resources.active_proxies import ActiveProxies
from stevesie.resources.proxy import Proxy
from stevesie.resources.task import Task
from stevesie.resources.task_collection import TaskCollection
from stevesie.resources.task_collection_field import TaskCollectionField
from stevesie.resources.task_collection_result import TaskCollectionResult
from stevesie.resources.task_collection_result_set import TaskCollectionResultSet
from stevesie.resources.task_dependency import TaskDependency
from stevesie.resources.worker import Worker
from stevesie.resources.worker_collection_results import WorkerCollectionResults
from stevesie.resources.workflow import Workflow
from stevesie.resources.workflow_parameter import WorkflowParameter
from stevesie.resources.workflow_task import WorkflowTask



from typing import NamedTuple, Sequence
from datetime import datetime

from stevesie.remote_resource import RemoteResource
from stevesie.resources.task_dependency import TaskDependency
from stevesie.resources.task_collection import TaskCollection

class TaskFields(NamedTuple):
    id: str
    name: str
    description: str
    is_public: bool
    slug: str
    method: str
    scheme: str
    host: str
    port: int
    path: str
    body: str
    created_at: datetime
    task_dependencies: Sequence[TaskDependency]
    task_collections: Sequence[TaskCollection]

TaskFields.__new__.__defaults__ = (None,) * len(TaskFields._fields)

class Task(TaskFields, RemoteResource):

    @property
    def resource_path(self):
        assert self.id
        return 'tasks/{}'.format(self.id)



import logging
import time

from typing import NamedTuple
from datetime import datetime

from stevesie.utils import api

from stevesie.remote_resource import RemoteResource
from stevesie.resources.task import Task
from stevesie.resources.workflow import Workflow
from stevesie.resources.worker_collection_results import WorkerCollectionResults

class WorkerFields(NamedTuple):
    id: str
    name: str
    created_at: datetime
    task: Task
    workflow: Workflow

WorkerFields.__new__.__defaults__ = (None,) * len(WorkerFields._fields)

class Worker(WorkerFields, RemoteResource):

    def run_params(self):
        if self.task:
            return self.task.task_dependencies
        else:
            return self.workflow.workflow_parameters

    def run_param_by_name(self, name):
        return next(p for p in self.run_params() if p.name == name)

    def run(self, inputs=None, saveResults=True):
        inputs = inputs or {}
        resp_json, status_code = api.post(self.resource_url + '/runs', {'inputs': inputs, 'saveResults': saveResults})
        if status_code == 400:
            if resp_json['errors'].get('proxy'):
                logging.info('Please launch a proxy to run your worker.')
            return False
        return resp_json

    def fetch_results(self, num_retries=6):
        attempt = self.__build_worker_collection_results().fetch()
        if num_retries > 0 and len([1 for collection_result in attempt.to_json() if collection_result['total'] > 0]) == 0:
            logging.info('No results found, retrying worker...')
            time.sleep(5) # elastic may be indexing, give it time...
            return self.fetch_results(num_retries - 1)
        return attempt

    def load_results(self, local_filepath):
        return self.__build_worker_collection_results().load_from_file(local_filepath)

    @property
    def resource_path(self):
        assert self.id
        return 'workers/{}'.format(self.id)

    def __build_worker_collection_results(self):
        collection_results = WorkerCollectionResults()
        collection_results.worker_id = self.id
        return collection_results



import os
import re

import pytest

import requests_mock

from tests.mock_api.app import app as app_json
from tests.mock_api.platform import platform as platform_json

from tests.mock_api.task_dependency import task_dependency as task_dependency_json
from tests.mock_api.task_header import task_header as task_header_json

from tests.mock_api.task_collection_field import task_collection_field as task_collection_field_json
from tests.mock_api.task_collection import task_collection as task_collection_json

from tests.mock_api.task import task as task_json

from tests.mock_api.workflow_maintainer import workflow_maintainer as workflow_maintainer_json
from tests.mock_api.workflow_task import workflow_task as workflow_task_json
from tests.mock_api.workflow_parameter import workflow_parameter as workflow_parameter_json
from tests.mock_api.workflow_task_dependency import workflow_task_dependency \
    as workflow_task_dependency_json

from tests.mock_api.workflow import workflow as workflow_json

from tests.mock_api.user_vault_entry import user_vault_entry as user_vault_entry_json
from tests.mock_api.vault_entry import vault_entry as vault_entry_json
from tests.mock_api.work_extraction import work_extraction as work_extraction_json

from tests.mock_api.worker import worker as worker_json

from tests.mock_api.task_collection_result import task_collection_result \
    as task_collection_result_json
from tests.mock_api.worker_collection_result import worker_collection_result \
    as worker_collection_result_json

os.environ['STEVESIE_CONFIG_FILE'] = 'stevesie.test.cfg'

UUID_REGEX = '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}'

@pytest.fixture
def mock_api(worker_json, worker_collection_result_json):
    mock = requests_mock.mock()

    mock.get(re.compile('/api/v1/workers/' + UUID_REGEX), json={'item': worker_json})
    mock.get(
        re.compile('/api/v1/workers/' + UUID_REGEX + '/collection-results'),
        json={'items': [worker_collection_result_json]})
    mock.get(
        re.compile('/api/v1/workers/' + UUID_REGEX + \
            '/collection-results\?taskCollectionId=' + UUID_REGEX + '&offset=\d+'),
        json={'item': worker_collection_result_json})
    mock.post(
        re.compile('/api/v1/workers/' + UUID_REGEX + \
            '/runs'),
        json={'item': {'taskResults': []}})

    return mock



from typing import NamedTuple
from datetime import datetime

from stevesie.remote_resource import RemoteResource

class ProxyTuple(NamedTuple):
    id: str
    hostname: str
    port: int
    location: str
    created_at: datetime

ProxyTuple.__new__.__defaults__ = (None,) * len(ProxyTuple._fields)

class Proxy(ProxyTuple, RemoteResource):
    pass



import logging

from stevesie.remote_resource_sequence import RemoteResourceSequence

from stevesie.utils import api
from stevesie.resources.proxy import Proxy

class ActiveProxies(RemoteResourceSequence):

    def launch(self, location='nyc3'):
        send_json = {
            'location': location,
            'blurImages': False
        }
        resp_json, status_code = api.post(api.BASE_URL + 'proxies', send_json)
        if status_code == 400:
            logging.info('Failed to launch proxy.')
            return False
        return self.hydrate([resp_json['item']])

    @property
    def collection_type(self):
        return Proxy

    @property
    def resource_path(self):
        return 'proxies/current'



from stevesie.utils import api

from stevesie import Worker

def new_worker_from_task_id(task_id):
    resp_json, status_code = api.post(api.BASE_URL + 'workers', {
        'workUnitType': 'task',
        'workUnitId': task_id})

    return Worker().hydrate(resp_json)

def temp_worker_from_task_id(task_id):
    pass



from typing import NamedTuple, Sequence
from datetime import datetime

from stevesie.remote_resource import RemoteResource
from stevesie.resources.task_collection_field import TaskCollectionField

class TaskCollectionTuple(NamedTuple):
    id: str
    name: str
    created_at: datetime
    child_task_collections: Sequence['TaskCollection']
    task_collection_fields: Sequence[TaskCollectionField]

TaskCollectionTuple.__new__.__defaults__ = (None,) * len(TaskCollectionTuple._fields)

class TaskCollection(TaskCollectionTuple, RemoteResource):
    pass



from typing import NamedTuple, Sequence
from datetime import datetime

from stevesie.remote_resource import RemoteResource
from stevesie.resources.workflow_parameter import WorkflowParameter
from stevesie.resources.workflow_task import WorkflowTask

class WorkflowFields(NamedTuple):
    id: str
    name: str
    description: str
    is_public: bool
    slug: str
    created_at: datetime
    workflow_parameters: Sequence[WorkflowParameter]
    workflow_tasks: Sequence[WorkflowTask]

WorkflowFields.__new__.__defaults__ = (None,) * len(WorkflowFields._fields)

class Workflow(WorkflowFields, RemoteResource):

    @property
    def resource_path(self):
        assert self.id
        return 'workflows/{}'.format(self.id)



import os
import re
import json

from typing import Sequence, GenericMeta
from datetime import datetime, date
import dateutil.parser

import inflection

from stevesie import resources
from stevesie.utils import api

DATETIME_FORMAT = '%Y-%m-%dT%H:%M:%S.%fZ'

class RemoteResource(object):

    _is_hydrated = False

    def set_hydrated(self):
        self._is_hydrated = True

    def hydrate(self, obj, fetch_remote=True):
        hydrate_args = {}
        for field_name in self._fields:
            field_type = self._field_types[field_name]
            api_field_name = inflection.camelize(field_name, uppercase_first_letter=False)
            field_value = obj.get(api_field_name, obj.get(field_name))

            if field_value is not None:
                if field_type == datetime:
                    field_value = dateutil.parser.parse(field_value)
                elif issubclass(field_type, RemoteResource):
                    field_value = field_type().hydrate(field_value, fetch_remote=fetch_remote)
                elif issubclass(field_type, Sequence) \
                    and issubclass(field_type.__class__, GenericMeta):
                    # TODO - serious debt, can't otherwise figure out the type of a typing.Sequence
                    sequence_class_string = str(field_type)

                    match = re.search(r'\[(.*)\]', sequence_class_string)
                    module_parts = match.group(1).split('.')

                    if len(module_parts) == 1: # referring to self using string type hack
                        class_name_match = re.search(r'\(\'(.*)\'\)', module_parts[0])
                        class_name = class_name_match.group(1)
                        module_name = inflection.underscore(class_name)
                    else:
                        module_name = module_parts[2]
                        class_name = module_parts[3]

                    mod = getattr(resources, module_name)
                    cls = getattr(mod, class_name)

                    field_value = [cls().hydrate(item, fetch_remote=fetch_remote) \
                        for item in field_value]

            hydrate_args[field_name] = field_value

        hydrated = self._replace(**hydrate_args)
        hydrated.set_hydrated()
        return hydrated

    def fetch(self):
        api_json = api.get(self.resource_url)
        obj = self.parse_api_response(api_json)
        return self.hydrate(obj)

    def destroy(self):
        api.delete(self.resource_url)

    def to_json(self, obj=None):
        if obj is None:
            obj = self

        def inner_json(inner_obj):
            if isinstance(inner_obj, list):
                return [self.to_json(o) for o in inner_obj]
            if isinstance(inner_obj, RemoteResource):
                return self.to_json(inner_obj)
            return inner_obj

        if hasattr(obj, 'collection_type') and obj.collection_type is not None:
            # little hack for implicit remote resource collection
            return [inner_json(value) for value in obj.items()]

        return {key: inner_json(value) for key, value in obj._asdict().items()}

    def save_to_file(self, local_filename):

        def serialize(obj):
            if isinstance(obj, (datetime, date)):
                return obj.strftime(DATETIME_FORMAT)
            raise TypeError('Cannot serialize %s' % type(obj))

        with open(os.path.expanduser(local_filename), 'w') as file:
            json.dump(self.to_json(), file, default=serialize)

    def load_from_file(self, local_filename):
        with open(os.path.expanduser(local_filename)) as file:
            obj = json.load(file)
        return self.hydrate(obj)

    def parse_api_response(self, api_json):
        return api_json['item']

    @property
    def resource_params(self):
        return {}

    @property
    def is_hydrated(self):
        return self._is_hydrated

    @property
    def resource_path(self):
        pass

    @property
    def resource_url(self):
        return api.BASE_URL + self.resource_path



from stevesie.remote_resource import RemoteResource
from stevesie.utils import api

class RemoteResourceSequence(RemoteResource):

    def __init__(self, *args, **kwargs):
        self._items = None
        super(RemoteResourceSequence, self).__init__(*args, **kwargs)

    def items(self):
        return self._items

    @property
    def collection_type(self):
        return None

    @property
    def collection_field(self):
        return None

    def fetch(self):
        api_json = api.get(self.resource_url)
        obj = self.parse_api_response(api_json)
        initial_hydration = self.hydrate(obj)
        return initial_hydration

    def hydrate(self, obj, fetch_remote=True):
        if self.collection_type:
            cls = self.collection_type
            self._items = [cls().hydrate(o, fetch_remote=fetch_remote) for o in obj]
            return self

        initial_hydration = super(RemoteResourceSequence, self).hydrate(obj)

        offset = 0
        while fetch_remote and self.collection_field \
            and len(getattr(initial_hydration, self.collection_field)) < initial_hydration.total:
            offset = offset + 10

            params = initial_hydration.resource_params
            params['offset'] = offset

            pagination_json = api.get(initial_hydration.resource_url, params)
            pagination_obj = self.parse_api_response(pagination_json)

            new_hydration = super(RemoteResourceSequence, self).hydrate(pagination_obj)

            merged_collection = getattr(initial_hydration, self.collection_field) \
                + getattr(new_hydration, self.collection_field)

            merged_args = {}
            merged_args[self.collection_field] = merged_collection

            initial_hydration = initial_hydration._replace(**merged_args)

        return initial_hydration

    def parse_api_response(self, api_json):
        if self.collection_type:
            return api_json['items']
        return super(RemoteResourceSequence, self).parse_api_response(api_json)



from typing import NamedTuple, Sequence

from stevesie.remote_resource_sequence import RemoteResourceSequence
from stevesie.resources.task_collection_result import TaskCollectionResult

class TaskCollectionResultSetTuple(NamedTuple):
    worker_id: str
    task_collection_id: str
    task_collection_results: Sequence[TaskCollectionResult]
    total: int

TaskCollectionResultSetTuple.__new__.__defaults__ = (None,) \
    * len(TaskCollectionResultSetTuple._fields)

class TaskCollectionResultSet(TaskCollectionResultSetTuple, RemoteResourceSequence):

    @property
    def results(self):
        return [o.object for o in self.task_collection_results]

    @property
    def collection_field(self):
        return 'task_collection_results'

    @property
    def resource_path(self):
        assert self.worker_id
        return 'workers/{}/collection-results'.format(self.worker_id)

    @property
    def resource_params(self):
        return {'taskCollectionId': self.task_collection_id}



from typing import NamedTuple, Sequence
from datetime import datetime

from stevesie.remote_resource import RemoteResource
from stevesie.resources.task_collection_field import TaskCollectionField

class TaskDependencyTuple(NamedTuple):
    id: str
    variable_name: str
    name: str
    sample_value: str
    created_at: datetime

TaskDependencyTuple.__new__.__defaults__ = (None,) * len(TaskDependencyTuple._fields)

class TaskDependency(TaskDependencyTuple, RemoteResource):
    pass



from typing import NamedTuple, Sequence
from datetime import datetime

from stevesie.remote_resource import RemoteResource

class WorkflowParameterTuple(NamedTuple):
    id: str
    name: str
    sample_value: str
    created_at: datetime

WorkflowParameterTuple.__new__.__defaults__ = (None,) * len(WorkflowParameterTuple._fields)

class WorkflowParameter(WorkflowParameterTuple, RemoteResource):
    pass



import uuid
import tempfile
import json

from stevesie import Worker

WORKER_ID = str(uuid.uuid4())

def test_init():
    worker = Worker(WORKER_ID)
    assert worker.id == WORKER_ID
    assert not worker.is_hydrated

def test_hydration(mock_api):
    worker = Worker(WORKER_ID)
    with mock_api:
        worker = worker.fetch()

    assert worker.is_hydrated
    assert worker.name == 'My custom worker!'

    with tempfile.NamedTemporaryFile() as temp_file:
        worker.save_to_file(temp_file.name)
        loaded_json = json.load(temp_file)
        assert loaded_json['name'] == 'My custom worker!'
        assert loaded_json['workflow']['name'] == 'Newsfeed'
        assert loaded_json['workflow']['workflow_tasks'][0]['id'] == \
            '83ca4ba0-73a6-49e9-99c8-71a0097fccd8'

def test_fetch_results(mock_api):
    worker = Worker(WORKER_ID)
    with mock_api:
        collection_results = worker.fetch_results()

    assert not worker.is_hydrated
    assert collection_results

def test_run_worker(mock_api):
    worker = Worker(WORKER_ID)
    with mock_api:
        run_result = worker.fetch().run()

    assert run_result



import setuptools

with open('README.md', 'r') as f:
    long_description = f.read()

setuptools.setup(
    name='stevesie',
    version='0.0.2',
    author='Stevesie, LLC',
    author_email='steve@stevesie.com',
    description='Stevesie Python Client.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/stevesie/stevesie-py',
    packages=setuptools.find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent'
    ],
    install_requires=[
        'requests>=0.11.1,<3.0',
        'inflection>=0.2.0,<1.0'
    ]
)



from typing import NamedTuple
from datetime import datetime

from stevesie.remote_resource import RemoteResource

class TaskCollectionResultTuple(NamedTuple):
    object: object
    task_result_id: str
    collected_at: datetime

TaskCollectionResultTuple.__new__.__defaults__ = (None,) * len(TaskCollectionResultTuple._fields)

class TaskCollectionResult(TaskCollectionResultTuple, RemoteResource):
    pass



import os
import configparser

CONFIG_FILE_ENV_KEY = 'STEVESIE_CONFIG_FILE'
DEFAULT_HOST = 'https://stevesie.com'

CONFIG = configparser.ConfigParser()
CONFIG.read(os.path.expanduser(os.getenv(CONFIG_FILE_ENV_KEY, '~/stevesie.cfg')))

API_TOKEN = CONFIG['core']['api_token']
BASE_HOST = CONFIG['core'].get('base_host', DEFAULT_HOST)

def worker_id_for(lookup_str):
    return CONFIG['workers'][lookup_str]



from stevesie.remote_resource_sequence import RemoteResourceSequence
from stevesie.resources.task_collection_result_set import TaskCollectionResultSet

class WorkerCollectionResults(RemoteResourceSequence):

    _worker_id = None

    @property
    def worker_id(self):
        return self._worker_id

    @worker_id.setter
    def worker_id(self, value):
        self._worker_id = value

    def for_task_collection_id(self, task_collection_id):
        return [r for r in self.items() if r.task_collection_id == task_collection_id][0]

    @property
    def collection_type(self):
        return TaskCollectionResultSet

    @property
    def resource_path(self):
        assert self.worker_id
        return 'workers/{}/collection-results'.format(self._worker_id)



from typing import NamedTuple
from datetime import datetime

from stevesie.remote_resource import RemoteResource
from stevesie.resources.task import Task

class WorkflowTaskTuple(NamedTuple):
    id: str
    created_at: datetime
    task: Task

WorkflowTaskTuple.__new__.__defaults__ = (None,) * len(WorkflowTaskTuple._fields)

class WorkflowTask(WorkflowTaskTuple, RemoteResource):
    pass



from typing import NamedTuple
from datetime import datetime

from stevesie.remote_resource import RemoteResource

class TaskCollectionFieldTuple(NamedTuple):
    id: str
    field_name: str
    field_type: str
    is_primary_key: bool
    created_at: datetime

TaskCollectionFieldTuple.__new__.__defaults__ = (None,) * len(TaskCollectionFieldTuple._fields)

class TaskCollectionField(TaskCollectionFieldTuple, RemoteResource):
    pass
